# -*- coding: utf-8 -*-

import numpy as np
import matplotlib.pyplot as plt
from ode_solvers import explicit_euler, implicit_euler, implicit_mid_point
from ode45 import ode45

def acceleration(t,y):
	eps = 0.0001
	omega = 0.01

	return -y + eps*np.cos(omega*t)

def ODE_rhs(t, y):
    """Rechte Seite der ODE.

       t : Aktuelle Zeit.
       y : 1D Array. Approximative Lösung der Differentialgleichung.
           y[0] : u.
           y[1] : Geschwindigkeit u'.
    """
    return np.array([y[...,1], acceleration(t, y[...,0])])

def convergence_rate(errors, resolutions):
    """ Berechnet die Konvergenzrate. """
    poly = np.polyfit(np.log(resolutions), np.log(errors), deg=1)
    return -poly[0]

def create_plots(t, y, filename):

    plt.plot(t, y, label=filename)
    plt.legend()

    plt.ylabel("Solution")
    plt.xlabel("Time")

def create_log_plots(t, y, filename):

    plt.loglog(t, y, label=filename)
    plt.legend()

    plt.ylabel("Error")
    plt.xlabel("Time")

def ex2d():
    T, N = 10.0, 100
    y0 = np.array([1., 0.0])

    all_methods = [explicit_euler, implicit_euler, implicit_mid_point]
    all_rhs = 3*[ODE_rhs]
    all_labels = ["ee", "ie", "im"]

    for method, rhs, label in zip(all_methods, all_rhs, all_labels):
        t, y = method(rhs, y0, T, N)
        create_plots(t,y[...,0],label)

    plt.savefig("methods_plot.eps")
    plt.show()

def ex2e():
    T = 1
    y0 = np.array([1., 0.0])

    all_methods = [explicit_euler, implicit_euler, implicit_mid_point]
    all_rhs = 3*[ODE_rhs]
    all_labels = ["ee", "ie", "im"]
    resolutions = 2**np.arange(4, 11)

    _, y_exact = ode45(ODE_rhs, (0.0, T), y0, reltol=1e-12)

    for method, rhs, label in zip(all_methods, all_rhs, all_labels):
        error = np.empty(resolutions.size)
        for k, N in enumerate(resolutions):
            _, y = method(rhs, y0, T, N)
            error[k] = np.abs(y[-1, 0] - y_exact[-1, 0])

        create_log_plots(resolutions,error,label)

        rate = convergence_rate(error, resolutions)
        print(method)
        print("rate: " + str(rate) + "\n")

    plt.savefig("methods_rate.eps")
    plt.show()

if __name__ == '__main__':
    
    ex2d()
    ex2e()
